# Copyright (c) 2003 Justin Balthrop and Dennis Chao
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use MP3::Info;
use KnuthRandom;

use Exporter ();
@ISA = qw(Exporter);

@EXPORT = qw(log_event);

use strict;
use vars qw($media_dir $etc_dir $history_length @files); 

require "/etc/adr/.perlvars";
$etc_dir = "/etc/adr";

# Songs will not be repeated in blocks of this size.
$history_length = 60;

sub ices_get_next {
    open(LOG, "> $etc_dir/log") 
	or die "error opening $etc_dir/log: $!";

    # Read in the playlist of all possible songs.
    open(LIST, "< $media_dir/playlist")
	or die "error opening $media_dir/playlist: $!";
    @files = <LIST>;
    close LIST;

    # Read in the history so we don't repeat songs too often.
    my $history = get_history();
    
    # Read in the censored and blocked songs of all users who are
    # logged in.
    my $censored;
    my $blocked;
    opendir(DIR, "$etc_dir/users");
    my $user;
    while (defined($user = readdir(DIR))) {
	if (-e "$etc_dir/users/$user/.login") {
	    open(CENSOR, "< $etc_dir/users/$user/censored_songs");
	    while (<CENSOR>) {
		chomp $_;
		push @$censored, $_;
	    }
	    close CENSOR;
	    open(BLOCK, "< $etc_dir/users/$user/blocked_songs");
	    while (<BLOCK>) {
		chomp $_;
		push @$blocked, $_;
	    }
	    close BLOCK;
	}
    }
    closedir DIR;

    # Keep going until we find a valid song to play.
    my $song;
    my $i;
    for ($i = 0; $i <= 2000; $i++) {
	if ($i < 1000) {
	    $song = random_song();
	    next if check_censors($song, $history, \&same_song);
	    next if check_censors($song, $blocked, \&same_song);
	    next if check_censors($song, $censored, \&same_album);
	    last;
	} elsif ($i < 2000) {
	    # If a song isn't found after 1000 tries, 
	    # don't worry about history.
	    $song = random_song();
	    next if check_censors($song, $censored, \&same_album);
	    last;
	} else {
	    # If a song isn't found after 1000 more tries, don't censor.
	    $song = random_song();
	}
    }


    # Write out the song that we have decided to play.
    open(FILE, "> $etc_dir/current_song")
	or die "error opening $etc_dir/current_song: $!";
    print FILE $song;
    close FILE;

    # Write out the updated history file.
    unshift @$history, $song;
    my $count = @$history;
    $count = $history_length if $count > $history_length; 
    open(HISTORY, "> $etc_dir/song_history") 
	or die "error opening $etc_dir/song_history: $!";
    for ($i = 0; $i < $count; $i++) {
	print HISTORY shift(@$history)."\n";
    }
    close HISTORY;
    close LOG;

    log_event('',"play $song");

    return "$media_dir/$song";
}

sub ices_get_metadata {
    my $song = current_song();
    my $tag = get_mp3tag("$media_dir/$song");

    return $tag->{ARTIST}." - ".$tag->{TITLE};
}

sub get_song_info {
    my $song = shift;
    my $tag = get_mp3tag("$media_dir/$song");

    return $tag->{ARTIST}." - ".$tag->{TITLE};
}

sub get_history {
    my $history;
    open(HISTORY, "< $etc_dir/song_history");
    while (<HISTORY>) {
	chomp $_;
	push @$history, $_;
    }
    close HISTORY;

    return $history;
}

sub random_song { 
    my $max = @files;
    my $num = int randk ($max);
    my $song = $files[$num];
    chomp $song;
    return $song;
}

sub check_censors {
    my $song           = shift;
    my $censor_list    = shift;
    my $match_function = shift;# || \&same_song;
    
    my $censor;
    foreach $censor (@$censor_list) {
	return 1 if $match_function->($song,$censor);
    }
    return 0;
}

sub same_song {
    my $song1 = shift;
    my $song2 = shift;

    return 1 if $song1 eq $song2;
}

sub same_artist {
    my $song1 = shift;
    my $song2 = shift;
    my $tag1 = get_mp3tag("$media_dir/$song1");
    my $tag2 = get_mp3tag("$media_dir/$song2");

    return 1 if $tag1->{ARTIST} eq $tag2->{ARTIST};
}

sub same_album {
    my $song1 = shift;
    my $song2 = shift;
    my $tag1 = get_mp3tag("$media_dir/$song1");
    my $tag2 = get_mp3tag("$media_dir/$song2");

    print LOG $tag1->{ALBUM}, $tag2->{ALBUM}, "\n";
    return 1 if $tag1->{ALBUM} eq $tag2->{ALBUM};
}

sub same_artist_album {
    my $song1 = shift;
    my $song2 = shift;
    my $tag1 = get_mp3tag("$media_dir/$song1");
    my $tag2 = get_mp3tag("$media_dir/$song2");

    return 1 if ($tag1->{ALBUM} eq $tag2->{ALBUM} &&
		    $tag1->{ARTIST} eq $tag2->{ARTIST});
}

sub current_song {
    open(FILE, "< $etc_dir/current_song");
    my $song = <FILE>;
    chomp $song;
    close FILE;
    return $song;
}

sub censor_song {
    my $user = shift;
    my $song = current_song();

    open(CENSORS, ">> $etc_dir/users/$user/censored_songs");
    print CENSORS "$song\n";
    close CENSORS;
    
    log_event($user, "censor $song");
}

sub block_song {
    my $user = shift;
    my $song = current_song();

    open(BLOCKS, ">> $etc_dir/users/$user/blocked_songs");
    print BLOCKS "$song\n";
    close BLOCKS;
    
    log_event($user, "block $song");
}


sub log_event {
    my $user = shift;
    my $event = shift;
    my $dir;
    if ($user) {
	$dir = "$etc_dir/users/$user";
    } else {
	$dir = "$etc_dir";
    }	    
    
    open(HISTORY, ">> $dir/.history");
    print HISTORY time, "\t$event\n";
    close HISTORY;
}

return 1;
